@@ -65,35 +65,40 @@
 #'     }
 #'
 #' @references
-#' Kook, L., Saengkyongam, S., Lundborg, A. R., Hothorn, T., & Peters, J. (2023).
-#' Model-based causal feature selection for general response types. arXiv
-#' preprint. \doi{10.48550/arXiv.2309.12833}
+#' Kook, L., Saengkyongam, S., Lundborg, A. R., Hothorn, T., & Peters, J.
+#' (2024). Model-based causal feature selection for general response types.
+#' Journal of the American Statistical Association, 1-12.
+#' \doi{10.1080/01621459.2024.2395588}
 #'
 #' @export
 #'
 #' @examples
 #' set.seed(12)
 #' d <- dgp_dicp(n = 1e3, mod = "binary")
-#' dicp(Y ~ X1 + X2 + X3, data = d, env = ~ E, modFUN = "glm",
-#'      family = "binomial", type = "wald")
+#' dicp(Y ~ X1 + X2 + X3,
+#'   data = d, env = ~E, modFUN = "glm",
+#'   family = "binomial", type = "wald"
+#' )
 #'
 dicp <- function(
-  formula, data, env, modFUN, verbose = TRUE,
-  type = c("residual", "wald", "partial"),
-  test = "gcm.test", controls = NULL, alpha = 0.05,
-  baseline_fixed = TRUE, greedy = FALSE, max_size = NULL,
-  mandatory = NULL, ...
-) {
-
+    formula, data, env, modFUN, verbose = TRUE,
+    type = c("residual", "wald", "partial"),
+    test = "gcm.test", controls = NULL, alpha = 0.05,
+    baseline_fixed = TRUE, greedy = FALSE, max_size = NULL,
+    mandatory = NULL, ...) {
   call <- match.call()
   type <- match.arg(type)
 
   ### Preliminary checks
-  if (is.character(test))
+  if (is.character(test)) {
     test <- match.arg(test, .implemented_tests())
-  if (is.null(controls))
-    controls <- dicp_controls(type, test, alpha = alpha,
-                              baseline_fixed = baseline_fixed)
+  }
+  if (is.null(controls)) {
+    controls <- dicp_controls(type, test,
+      alpha = alpha,
+      baseline_fixed = baseline_fixed
+    )
+  }
   .check_args(formula, data, env, modFUN, type, test)
 
   ### Process formulae
@@ -101,20 +106,23 @@ dicp <- function(
   resp <- tms$response
   etms <- .get_terms(env)
   me <- setdiff(tms$all, etms$all) # no env in main effects
-  if (is.null(max_size))
+  if (is.null(max_size)) {
     max_size <- length(me)
+  }
   max_size <- min(max_size, length(me))
   ps <- lapply(0:max_size, utils::combn, x = length(me))
 
   ### Options
-  if (verbose && interactive())
+  if (verbose && interactive()) {
     pb <- utils::txtProgressBar(min = 0, max = length(ps), style = 3)
+  }
 
   ### Run invariant subset search
   out <- .invariant_subset_search(
     ps = ps, controls = controls, me = me, resp = resp, etms = etms,
     modFUN = modFUN, data = data, greedy = greedy, verbose = verbose, pb = pb,
-    mandatory = mandatory, ... = ...)
+    mandatory = mandatory, ... = ...
+  )
 
   ### Process output
   tests <- out$tests
@@ -124,22 +132,28 @@ dicp <- function(
   pvals <- structure(res[["pval"]], names = res[["set"]])
   if (!greedy) {
     inv <- try(.inv_set(res, alpha = controls$alpha))
-    if (inherits(inv, "try-error"))
+    if (inherits(inv, "try-error")) {
       inv <- "Cannot be computed."
+    }
   } else {
     inv <- me[sort(unique(unlist(out$MI)))]
   }
 
   ### Compute predictor-level p-values
   ipv <- .indiv_pvals(me, pvals, alpha = alpha)
-  if (identical(inv, character(0)))
+  if (identical(inv, character(0))) {
     inv <- "Empty"
+  }
   ### Return
-  structure(list(candidate_causal_predictors = inv, set_pvals = pvals,
-                 predictor_pvals = ipv, tests = tests, controls = controls,
-                 call = call), class = "dICP", type = type,
-            test = controls$test_name, env = env, greedy = greedy)
-
+  structure(
+    list(
+      candidate_causal_predictors = inv, set_pvals = pvals,
+      predictor_pvals = ipv, tests = tests, controls = controls,
+      call = call
+    ),
+    class = "dICP", type = type,
+    test = controls$test_name, env = env, greedy = greedy
+  )
 }
 
 #' Return invariant sets
@@ -156,8 +170,9 @@ dicp <- function(
 invariant_sets <- function(object, with_pvalues = FALSE) {
   pvals <- pvalues(object, "set")
   ret <- pvals[pvals > object$controls$alpha]
-  if (with_pvalues)
+  if (with_pvalues) {
     return(ret)
+  }
   names(ret)
 }
 
@@ -166,30 +181,31 @@ invariant_sets <- function(object, with_pvalues = FALSE) {
 .invariant_subset_search <- function(ps, controls, me, resp, etms, modFUN,
                                      data, greedy, verbose, pb, mandatory,
                                      ...) {
-
   if (!greedy) {
     ### Run
     tests <- list()
     MI <- NULL
     for (set in seq_along(ps)) {
-
-      if (verbose && interactive())
+      if (verbose && interactive()) {
         utils::setTxtProgressBar(pb, set)
+      }
 
-      ret <- apply(ps[[set]], 2, controls$type_fun, me = me, resp = resp,
-                   set = set, env = etms, modFUN = modFUN, data = data,
-                   controls = controls, mandatory = mandatory, ... = ...)
+      ret <- apply(ps[[set]], 2, controls$type_fun,
+        me = me, resp = resp,
+        set = set, env = etms, modFUN = modFUN, data = data,
+        controls = controls, mandatory = mandatory, ... = ...
+      )
 
       if (set == 1 && controls$stop_if_empty_set_invariant &&
-          .get_pvalue(ret[[1]]$test) > controls$alpha) {
-      if (verbose && interactive())
-        message("\nEmpty set is not rejected. Stopping.")
+        .get_pvalue(ret[[1]]$test) > controls$alpha) {
+        if (verbose && interactive()) {
+          message("\nEmpty set is not rejected. Stopping.")
+        }
         tests <- ret
         break
       }
 
       tests <- c(tests, ret)
-
     }
   } else {
     ### Run
@@ -197,9 +213,9 @@ invariant_sets <- function(object, with_pvalues = FALSE) {
     MI <- list()
     lps <- .unlist_once(lapply(ps, \(x) apply(x, 2, \(y) y, simplify = FALSE)))
     for (set in seq_along(lps)) {
-
-      if (verbose && interactive())
+      if (verbose && interactive()) {
         utils::setTxtProgressBar(pb, set)
+      }
 
       if (length(MI > 0) && any(unlist(MI) %in% lps[[set]])) {
         tests[[set]] <- .empty_output(me[lps[[set]]], 0)
@@ -207,7 +223,8 @@ invariant_sets <- function(object, with_pvalues = FALSE) {
       }
 
       ret <- controls$type_fun(
-        lps[[set]], me = me, resp = resp, set = set, env = etms,
+        lps[[set]],
+        me = me, resp = resp, set = set, env = etms,
         modFUN = modFUN, data = data, controls = controls,
         mandatory = mandatory, ... = ...
       )
@@ -217,14 +234,13 @@ invariant_sets <- function(object, with_pvalues = FALSE) {
       }
 
       UMI <- sort(unique(unlist(MI)))
-      if (length(MI) > 0 && length(UMI) == length(me) && all(UMI == seq_along(me)))
+      if (length(MI) > 0 && length(UMI) == length(me) && all(UMI == seq_along(me))) {
         break
+      }
 
       tests[[set]] <- ret
-
     }
   }
 
   list(tests = tests, MI = MI)
 }
-
